% ------------------------------------------------------------------- %
%             Portfolio Return & Risk (In Class)                      %
% ------------------------------------------------------------------- %

clearvars; close all; clc;

%% === Load Price Data ===
filename = "Dataset.xlsx";
sheet = "Stock Prices";

opts = detectImportOptions(filename, 'Sheet', sheet);
%opts = setvartype(opts, 1, 'datetime');
opts = setvartype(opts, 2:numel(opts.VariableNames), 'double');
opts = setvaropts(opts, 2:numel(opts.VariableNames), 'TreatAsMissing', 'NaN');

T = readtable(filename, opts);
D = T{:, 1};
Names = T.Properties.VariableNames(2:end);
P = table2array(T(:, 2:end));

%% === Grafico delle due serie (Asset 1 and 40) ===
figure(1);
plot(D, P(:, [1 40]));
legend(Names{[1 40]});
title('Stocks 1 and 40 - Prices');
xlabel('Date'); ylabel('Price');

%% === Calcolo dei rendimenti ===
%R = 100 * diff(P) ./ P(1:end-1, :);
R = 100*((P(2:end,:)-P(1:end-1,:))./P(1:end-1,:));
D_R = D(2:end);

% Estraggo asset 1 e 40
R1 = R(:, 1);
R40 = R(:, 40);

%% === Grafico dei rendimenti ===
figure(2);
plot(D_R, R1);
hold on;
plot(D_R, R40, 'r');
legend('R_1', 'R_{40}');
title('Stocks 1 and 40 - Returns');
xlabel('Date'); ylabel('Returns');
hold off;

%% === Rischio e Rendimento di portafoglio: 3 Metodi ===
w = [0.7; 0.3];
C_2 = cov(R1, R40);

% 1. Formula analitica - teorico
mu_1 = w(1)*mean(R1) + w(2)*mean(R40);
Vp_1 = (w(1)^2)*var(R1) + (w(2)^2)*var(R40) + 2*w(1)*w(2)*C_2(1,2);
Sp_1 = sqrt(Vp_1);

% 2. Media della serie dei rendimenti di portafoglio - storico
Rp_2 = w(1)*R1 + w(2)*R40;
mu_2 = mean(Rp_2);
Sp_2 = std(Rp_2);

% 3. Notazione matriciale
mu_3 = [mean(R1), mean(R40)] * w;
Sp_3 = sqrt(w' * cov([R1 R40]) * w);

%% === Portafoglio con 10 Assets ===
R10 = R(:, 1:10);

% Metodo 1: media della serie dei rendimenti - approccio storico
Rp_10 = mean(R10, 2, 'omitnan');  % media per riga (serie rendimenti)
Rp_4 = mean(Rp_10);
Sp_4 = std(Rp_10);

% Metodo 2: media dei rendimenti attesi - approccio teorico
meanR_10 = mean(R10, 1, 'omitnan');  % media per colonna (media asset)
Rp_5 = mean(meanR_10);
Sp_5 = std(meanR_10, 'omitnan');

%% Frontiera efficiente con 2 asset

% Selezione di due asset per costruire la frontiera
assets = [494 268];
Port = R(:, assets);
%Names = T.Properties.VariableNames(assets + 1);  % +1 per la colonna data

% Calcolo di rendimento atteso e matrice di covarianza
mu = mean(Port)';  % Rendimento atteso di ciascun asset
Sigma = cov(Port); % Covarianza tra i due asset

%calcolo del rendrimento atteso e deviazione standard per un portafoglio
%specific
w_s = [0.3; 0.7];
mu_s = mu' * w_s;
sigma_s = sqrt(w_s' * Sigma * w_s);

% Costruzione di portafogli con pesi diversi
N = 100;  % Numero di portafogli
w1 = linspace(0,1,N);  % Pesi del primo asset
w2 = 1 - w1;           % Complementare
w = [w1; w2];    % Matrice dei pesi (2 x N)

% Calcolo rendimento atteso dei portafogli
mu_p = mu' * w;  % (1 x 2) * (2 x N) = (1 x N)

% Calcolo del rischio dei portafogli
sigma_p = zeros(N, 1);
for i = 1:N
    w_i = w(:, i);
    sigma_p(i) = sqrt(w_i' * Sigma * w_i);
end

%sigma_p2 = sqrt(sum((w' * Sigma) .* w', 2));  % (N x 1)

% Visualizzazione della frontiera
figure(3);
plot(sigma_p, mu_p, 'b-', 'LineWidth', 2); %Frontiera 
hold on;
plot(sqrt(diag(Sigma)), mu, 'ro', 'MarkerSize', 8); %Singoli Asset

% Evidenzia un portafoglio specifico
plot(sigma_s, mu_s, 'kp', 'MarkerSize', 12, 'MarkerFaceColor', 'k'); %portafoglio scelto

xlabel('\sigma (Rischio)');
ylabel('\mu (Rendimento atteso)');
title('Frontiera Efficiente (2 Asset)');
legend('Frontiera Efficiente', 'Singoli Asset', 'Portafoglio scelto', 'Location', 'best');
grid on;
hold off
%% Creare una funzione e plottare diverse correlazioni
%calcolo la correlazione tra i due titoli
corr_tit = corr(Port);
%compongo un vettore con le correlazioni da graficare
corr_dy =[-1 1 corr_tit(2,1)]; 



meanPort = zeros(3,100); %preallocazione variabili
stdPort = zeros(3,100); %preallocazione variabili
for j = 1:3 %ciclo sulle 3 correlazioni da graficare
    [meanPort(j,:),stdPort(j,:)] = ComputeFrontier(Port, corr_dy(j)); %richiamo la funzione creata
end

figure(4)
plot(stdPort(1,:), meanPort(1,:), 'b', 'LineWidth', 2); %Frontiera
hold on
plot(stdPort(2,:), meanPort(2,:), 'b-', 'LineWidth', 2); %Frontiera 
plot(stdPort(3,:), meanPort(3,:), 'g-', 'LineWidth', 2); %Frontiera 
plot(sqrt(diag(Sigma)), mu,'m.','MarkerSize',20);
plot(sigma_s,mu_s,'kp','MarkerSize', 12, 'MarkerFaceColor', 'k');
xlabel('\sigma')
ylabel('R')
title('Efficient Frontier')
legend('\rho = -1','\rho = 1',['\rho =' num2str(corr_dy(3))] ,'Single Stocks','Portfolio w_1 = 30%, w_2 = 70%','Location','northwest')
hold off