%% 
% --------------------------------------------------------------------- %
%                 Estimate the Market Model (in class)                  %
% --------------------------------------------------------------------- %


clear variables
clc

% Carico la matrice dei prezzi, i extra rendimento di mercato,i rendimenti
% risk-free
P  = readmatrix('Dataset.xlsx','Sheet','Stock Prices','Range','B2:SJ3274');
EMkt  = readmatrix('Dataset.xlsx','Sheet','Factors','Range','B2:B3273');
Rf   = readmatrix('Dataset.xlsx','Sheet','Factors','Range','F2:F3273');

EMkt = flipud(EMkt);%invertiamo l'ordine dei dati per renderlo coerente con i Prezzi
Rf   = flipud(Rf); 

Mkt = EMkt + Rf;%rendimento di mercato ricordando che Emkt = Mkt - Rf

% Calcoliamo i rendimenti percentuali e selezioniamo l'asset 494 e 268
R = 100*(P(2:end,:)-P(1:end-1,:))./P(1:end-1,:);
R_494 = R(:,494);
R_268 = R(:,268);

%controlliamo che non ci siano nan
chk494 = anynan(R_494);
chk268 = anynan(R_268);


% scatter plot 
 
figure(1)
subplot(1,2,1);
scatter(Mkt,R_494);
xlabel('R_{m}');
ylabel('R_{494}');
title('Market Model - Asset 494');
ylim([-50 50]) %per avere la stessa scala su i due grafici
h=lsline;
set(h,'color','r');
subplot(1,2,2);
scatter(Mkt,R_268);
xlabel('R_{m}');
ylabel('R_{268}');
title('Market Model - Asset 286');
h=lsline;
set(h,'color','r');

% Stimiamo Market Model for ogni asset


sim = fitlm(Mkt,R_494);
b_494 = sim.Coefficients.Estimate;
sim = fitlm(Mkt,R_268);
b_268 = sim.Coefficients.Estimate;


% il beta si può calcolare anche come:
c = cov(R_494,Mkt);
beta_494 = c(1,2)/var(Mkt);
alpha_494 = mean(R_494) - beta_494*mean(Mkt);

%stimiamo il  market model per ogni asset del dataset
[T, N] = size(R);

b2 = nan(N,2);%preallocazione
for i = 1:N
    if not(anynan(R(:,i))) %controlliamo i NAN
        Y = R(:,i);
        b2(i,:) = fitlm(Mkt,Y).Coefficients.Estimate;
    end
end

% Stimamo il Market Model per un portafoglio 
w_1 = 0.6;
w_2 = 0.4;

%Primo Metodo (NB: i due asset selezionati non hanno nan)
bp_t = w_1*b_494+w_2*b_268;


% Market Beta Ricorsivo
W = 20;%finestra di stima
Port = [494 268];%selezione asset
Rp = R(:,Port); 

B_p = NaN(T-W+1,length(Port));
A_p = NaN(T-W+1,length(Port));
for i = 1:T-W+1 %ciclo sul tempo
    for j = 1:length(Port) %ciclo sugli asset
        EstWind = i:W+i-1; %definizione indici della finestra di stima
        Y = Rp(EstWind,j);%selezione della finestra di stima per Y
        X =  Mkt(EstWind);%selezione della finestra di stima per X
        if not(anynan(Y))
            B_temp = fitlm(X,Y).Coefficients.Estimate;
            B_p(i,j) = B_temp(1);
            A_p(i,j) = B_temp(2);
        end 
    end
end

%calcolo alpha e beta del porafoglio
Bport = B_p*[w_1; w_2]; 
Aport = A_p*[w_1; w_2]; 



%plot del time-varying beta
Time = 1:size(B_p,1);

figure(2)
plot(Time,Bport);
xlabel('Time');
ylabel('\beta_{pm}');
title('Market Model');
xlim([20 3272])


%plot del time-varying alpha
figure(3)
plot(Time,Aport);
xlabel('Time');
ylabel('\alpha_{pm}');
title('Market Model');
xlim([20 3272])
